﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Text;
using BusinessModel;
using Infrastructure;
using Interfaces.Services;
using Microsoft.Practices.EnterpriseLibrary.Validation;

namespace DataRepository
{
  public abstract class BaseRepository<T> : IBaseRepository where T : DbContext
  {

    internal ILogger LoggerSvc
    { get { return _loggerSvc ?? (_loggerSvc = ServiceLocator.Resolve<ILogger>()); } }
    private ILogger _loggerSvc;

    internal ISecurityService SecuritySvc
    { get { return _securitySvc ?? (_securitySvc = ServiceLocator.Resolve<ISecurityService>()); } }
    private ISecurityService _securitySvc;

    /// <summary>
    /// Instance of the currently registered IValidationService, lazily fetched from the ServiceLocator
    /// </summary>
    internal IValidationService ValidationService
    { get { return _validationSvc ?? (_validationSvc = ServiceLocator.Resolve<IValidationService>()); } }
    private IValidationService _validationSvc;

    protected internal T DbContext { get; set; }

    /// <summary>
    /// Internal string used for connecting to the database
    /// </summary>
    internal string _connectionString { get; set; }

    public BaseRepository()
    {
    }

    /// <summary>
    /// Called just before items are saved
    /// Allows descendents to add custom logic
    /// </summary>
    protected virtual void BeforeSavingChanges()
    {
    }

 
    /// <summary>
    /// Called just after items are saved.
    /// Allows descendents to add custom logic
    /// </summary>
    protected virtual void AfterSavingChanges()
    {
    }

    /// <summary>
    /// Verifies all edited items, then saves 
    /// all pending changes to the database
    /// </summary>
    public ValidationResults SaveChanges()
    {
      try
      {
        //Validate data and exit if invalid
        var results = Validate();
        if (!results.IsValid) return results;

        //Update the UpdateDate and UpdateBy fields
        UpdateStandardFields();

        //Save changes to DB
        DbContext.SaveChanges();

        return results;
      }
      catch (Exception e)
      {
        LoggerSvc.Log(e);

        //Change to appropriate logic
        throw;
      }
    }

    /// <summary>
    /// Updates version tracking and other fields that need to be changed before a save
    /// </summary>
    internal void UpdateStandardFields()
    {
      var lst = GetUpdatedItems();
      foreach (var ent in lst)
      {
        ent.UpdatedBy = SecuritySvc.CurrentUsername;
        ent.UpdatedDate = DateTime.Now;
      }
    }

    /// <summary>
    /// Validates all changed entities in this Repository
    /// </summary>
    public ValidationResults Validate()
    {
      var entities = GetUpdatedItems();
      return ValidationService.Validate(entities);
    }

    /// <summary>
    /// Returns a list of all updated entities
    /// </summary>
    internal List<BaseEntity> GetUpdatedItems()
    {
      var allEntities = DbContext.ChangeTracker.Entries()
        .Where(e => e.State == EntityState.Added 
          || e.State == EntityState.Modified)
        .Select(e => e.Entity)
        .OfType<BaseEntity>().ToList();
      return allEntities;
    }

    public void CancelChanges()
    {
      var entries = DbContext.ChangeTracker.Entries().Where(e => e.State != EntityState.Unchanged);
      foreach (var dbEntityEntry in entries)
      {
        if (dbEntityEntry.State == EntityState.Added)
        {
          ((IObjectContextAdapter)DbContext).ObjectContext.Detach(dbEntityEntry.Entity);
        }
        else
        {

          dbEntityEntry.Reload();
        }
      }
    }
  }
}
